<?php
/**
 * @package   ats
 * @copyright Copyright (c)2011-2014 Nicholas K. Dionysopoulos / AkeebaBackup.com
 * @license   GNU GPL v3 or later
 */

// No direct access
defined('_JEXEC') or die;

class AtsModelCpanels extends FOFModel
{
	/** @var string The root of the database installation files */
	private $dbFilesRoot = '/components/com_ats/sql/';

	/** @var array If any of these tables is missing we run the install SQL file and ignore the $dbChecks array */
	private $dbBaseCheck = array(
		'tables' => array(
			'ats_tickets', 'ats_posts', 'ats_attachments',
		),
		'file'   => 'install/mysql/install.sql'
	);

	/** @var array Database update checks */
	private $dbChecks = array(
		array(
			'table' => 'ats_managernotes',
			'field' => '',
			'files' => array(
				'updates/mysql/1.0.0-2012-07-12.sql',
			)
		),

		array(
			'table' => 'ats_cannedreplies',
			'field' => '',
			'files' => array(
				'updates/mysql/1.0.0-2012-07-12.sql',
			)
		),

		array(
			'table' => 'ats_emailtemplates',
			'field' => '',
			'files' => array(
				'updates/mysql/1.0.0-2012-10-31.sql',
			)
		),

		array(
			'table' => 'ats_managernotes',
			'field' => 'note_html',
			'files' => array(
				'updates/mysql/1.0.0-2012-10-31.sql',
			)
		),

		array(
			'table' => 'ats_posts',
			'field' => 'origin',
			'files' => array(
				'updates/mysql/1.0.0-2012-10-31.sql',
			)
		),

		array(
			'table' => 'ats_posts',
			'field' => 'email_uid',
			'files' => array(
				'updates/mysql/1.0.0-2012-10-31.sql',
			)
		),

		array(
			'table' => 'ats_attachments',
			'field' => 'origin',
			'files' => array(
				'updates/mysql/1.0.0-2012-10-31.sql',
			)
		),

		array(
			'table' => 'ats_credittransactions',
			'field' => '',
			'files' => array(
				'updates/mysql/1.0.0-2012-11-12.sql',
			)
		),

		array(
			'table' => 'ats_creditconsumptions',
			'field' => '',
			'files' => array(
				'updates/mysql/1.0.0-2012-11-12.sql',
			)
		),

		array(
			'table' => 'ats_offlineschedules',
			'field' => '',
			'files' => array(
				'updates/mysql/1.0.0-2013-03-07.sql',
			)
		),

		array(
			'table' => 'ats_tickets',
			'field' => 'timespent',
			'files' => array(
				'updates/mysql/1.0.0-2013-04-03.sql',
			)
		),

		array(
			'table' => 'ats_posts',
			'field' => 'timespent',
			'files' => array(
				'updates/mysql/1.0.0-2013-04-03.sql',
			)
		),

		array(
			'table' => 'ats_tickets',
			'field' => 'assigned_to',
			'files' => array(
				'updates/mysql/1.1.0-2013-04-08.sql',
			)
		),

		array(
			'table' => 'ats_tickets',
			'field' => 'params',
			'files' => array(
				'updates/mysql/1.1.0-2013-04-25.sql',
			)
		),

		array(
			'table' => 'ats_customfields',
			'field' => '',
			'files' => array(
				'updates/mysql/1.1.0-2013-04-25.sql',
			)
		),

		array(
			'table' => 'ats_customfields_cats',
			'field' => '',
			'files' => array(
				'updates/mysql/1.1.0-2013-04-25.sql',
			)
		),

		array(
			'table' => 'ats_tickets',
			'field' => 'ats_bucket_id',
			'files' => array(
				'updates/mysql/1.1.0-2013-05-03.sql',
			)
		),

		array(
			'table' => 'ats_buckets',
			'field' => '',
			'files' => array(
				'updates/mysql/1.1.0-2013-05-03.sql',
			)
		),

		array(
			'table' => 'ats_tickets',
			'field' => 'priority',
			'files' => array(
				'updates/mysql/1.1.0-2013-05-08.sql',
			)
		),

		array(
			'table' => 'ats_tickets',
			'field' => 'rating',
			'files' => array(
				'updates/mysql/1.2.0-2013-06-05.sql',
			)
		),
	);

	/**
	 * Checks the database for missing / outdated tables using the $dbChecks
	 * data and runs the appropriate SQL scripts if necessary.
	 *
	 * @return AkeebasubsModelCpanels
	 */
	public function checkAndFixDatabase()
	{
		$db = $this->getDbo();

		// Initialise
		$tableFields = array();
		$sqlFiles = array();

		// Get a listing of database tables known to Joomla!
		$allTables = $db->getTableList();
		$dbprefix = JFactory::getConfig()->get('dbprefix', '');

		// Perform the base check. If any of these tables is missing we have to run the installation SQL file
		if (!empty($this->dbBaseCheck))
		{
			foreach ($this->dbBaseCheck['tables'] as $table)
			{
				$tableName = $dbprefix . $table;
				$check = in_array($tableName, $allTables);
				if (!$check)
				{
					break;
				}
			}

			if (!$check)
			{
				$sqlFiles[] = JPATH_ADMINISTRATOR . $this->dbFilesRoot . $this->dbBaseCheck['file'];
			}
		}

		// If the base check was successful and we have further database checks run them
		if (empty($sqlFiles) && !empty($this->dbChecks))
		{
			foreach ($this->dbChecks as $dbCheck)
			{
				// Always check that the table exists
				$tableName = $dbprefix . $dbCheck['table'];
				$check = in_array($tableName, $allTables);

				// If the table exists and we have a field, check that the field exists too
				if (!empty($dbCheck['field']) && $check)
				{
					if (!array_key_exists($tableName, $tableFields))
					{
						$tableFields[$tableName] = $db->getTableColumns('#__' . $dbCheck['table'], true);
					}

					if (is_array($tableFields[$tableName]))
					{
						$check = array_key_exists($dbCheck['field'], $tableFields[$tableName]);
					}
					else
					{
						$check = false;
					}
				}

				// Something's missing. Add the file to the list of SQL files to run
				if (!$check)
				{
					foreach ($dbCheck['files'] as $file)
					{
						$sqlFiles[] = JPATH_ADMINISTRATOR . $this->dbFilesRoot . $file;
					}
				}
			}
		}

		// If we have SQL files to run, well, RUN THEM!
		if (!empty($sqlFiles))
		{
			JLoader::import('joomla.filesystem.file');
			foreach ($sqlFiles as $file)
			{
				$sql = JFile::read($file);
				if ($sql)
				{
					$commands = explode(';', $sql);
					foreach ($commands as $query)
					{
						$db->setQuery($query);
						$db->execute();
					}
				}
			}

			// Finally clear FOF's cache
			$fof_platform = FOFPlatform::getInstance();
			if (method_exists($fof_platform, 'clearCache'))
			{
				FOFPlatform::getInstance()->clearCache();
			}
		}

		return $this;
	}

	/**
	 * Does the user needs to enter a Download ID?
	 *
	 * @return boolean
	 */
	public function needsDownloadID()
	{
		JLoader::import('joomla.application.component.helper');

		// Do I need a Download ID?
		$ret = false;
		$params = JComponentHelper::getParams('com_ats');
		if (version_compare(JVERSION, '3.0', 'ge'))
		{
			$dlid = $params->get('downloadid', '');
		}
		else
		{
			$dlid = $params->getValue('downloadid', '');
		}
		if (!preg_match('/^([0-9]{1,}:)?[0-9a-f]{32}$/i', $dlid))
		{
			$ret = true;
		}

		return $ret;
	}

	/**
	 * Refreshes the Joomla! update sites for this extension as needed
	 *
	 * @return  void
	 */
	public function refreshUpdateSite()
	{
		$isPro = defined('ATS_PRO') ? ATS_PRO : 0;

		JLoader::import('joomla.application.component.helper');
		$params = JComponentHelper::getParams('com_ats');

		if(version_compare(JVERSION, '3.0', 'ge'))
		{
			$dlid = $params->get('downloadid', '');
		}
		else
		{
			$dlid = $params->getValue('downloadid', '');
		}

		$extra_query = null;

		// If I have a valid Download ID I will need to use a non-blank extra_query in Joomla! 3.2+
		if (preg_match('/^([0-9]{1,}:)?[0-9a-f]{32}$/i', $dlid))
		{
			// Even if the user entered a Download ID in the Core version. Let's switch his update channel to Professional
			$isPro = true;

			$extra_query = 'dlid=' . $dlid;
		}

		// Create the update site definition we want to store to the database
		$update_site = array(
			'name'		=> 'Akeeba Ticket System ' . ($isPro ? 'Professional' : 'Core'),
			'type'		=> 'extension',
			'location'	=> 'http://cdn.akeebabackup.com/updates/ats' . ($isPro ? '' : 'core') . '.xml',
			'enabled'	=> 1,
			'last_check_timestamp'	=> 0,
			'extra_query'	=> $extra_query
		);

		if (version_compare(JVERSION, '3.0.0', 'lt'))
		{
			unset($update_site['extra_query']);
		}

		$db = $this->getDbo();

		// Get the extension ID to ourselves
		$query = $db->getQuery(true)
			->select($db->qn('extension_id'))
			->from($db->qn('#__extensions'))
			->where($db->qn('type') . ' = ' . $db->q('component'))
			->where($db->qn('element') . ' = ' . $db->q('com_ats'));
		$db->setQuery($query);

		$extension_id = $db->loadResult();

		if (empty($extension_id))
		{
			return;
		}

		// Get the update sites for our extension
		$query = $db->getQuery(true)
			->select($db->qn('update_site_id'))
			->from($db->qn('#__update_sites_extensions'))
			->where($db->qn('extension_id') . ' = ' . $db->q($extension_id));
		$db->setQuery($query);

		$updateSiteIDs = $db->loadColumn(0);

		if (!count($updateSiteIDs))
		{
			// No update sites defined. Create a new one.
			$newSite = (object)$update_site;
			$db->insertObject('#__update_sites', $newSite);

			$id = $db->insertid();

			$updateSiteExtension = (object)array(
				'update_site_id'	=> $id,
				'extension_id'		=> $extension_id,
			);
			$db->insertObject('#__update_sites_extensions', $updateSiteExtension);
		}
		else
		{
			// Loop through all update sites
			foreach ($updateSiteIDs as $id)
			{
				$query = $db->getQuery(true)
					->select('*')
					->from($db->qn('#__update_sites'))
					->where($db->qn('update_site_id') . ' = ' . $db->q($id));
				$db->setQuery($query);
				$aSite = $db->loadObject();

				// Does the name and location match?
				if (($aSite->name == $update_site['name']) && ($aSite->location == $update_site['location']))
				{
					// Do we have the extra_query property (J 3.2+) and does it match?
					if (property_exists($aSite, 'extra_query'))
					{
						if ($aSite->extra_query == $update_site['extra_query'])
						{
							continue;
						}
					}
					else
					{
						// Joomla! 3.1 or earlier. Updates may or may not work.
						continue;
					}
				}

				$update_site['update_site_id'] = $id;
				$newSite = (object)$update_site;
				$db->updateObject('#__update_sites', $newSite, 'update_site_id', true);
			}
		}
	}

	/**
	 * Checks if the download ID provisioning plugin for the updates of this extension is published. If not, it will try
	 * to publish it automatically. It reports the status of the plugin as a boolean.
	 *
	 * @return  bool
	 */
	public function isUpdatePluginEnabled()
	{
		// We can't be bothered about the plugin in Joomla! 2.5.0 through 2.5.19
		if (version_compare(JVERSION, '2.5.19', 'lt'))
		{
			return true;
		}

		// We can't be bothered about the plugin in Joomla! 3.x
		if (version_compare(JVERSION, '3.0.0', 'gt'))
		{
			return true;
		}

		$db = $this->getDBO();

		// Let's get the information of the update plugin
		$query = $db->getQuery(true)
			->select('*')
			->from($db->qn('#__extensions'))
			->where($db->qn('folder').' = '.$db->quote('installer'))
			->where($db->qn('element').' = '.$db->quote('akeebaticketsystem'))
			->where($db->qn('type').' = '.$db->quote('plugin'))
			->order($db->qn('ordering').' ASC');
		$db->setQuery($query);
		$plugin = $db->loadObject();

		// If the plugin is missing report it as unpublished (of course!)
		if (!is_object($plugin))
		{
			return false;
		}

		// If it's enabled there's nothing else to do
		if ($plugin->enabled)
		{
			return true;
		}

		// Otherwise, try to enable it and report false (so the user knows what he did wrong)
		$pluginObject = (object)array(
			'extension_id'	=> $plugin->extension_id,
			'enabled'		=> 1
		);

		try
		{
			$result = $db->updateObject('#__extensions', $pluginObject, 'extension_id');
			// Do not remove this line. We need to tell the user he's doing something wrong.
			$result = false;
		}
		catch (Exception $e)
		{
			$result = false;
		}

		return $result;
	}
}